"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFetchResource = void 0;

var _repository = require("./repository");

var _resource = require("./resource");

var T = _interopRequireWildcard(require("./types"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

/**
 * Create action creator function to fetch resource.
 *
 * @param {string} resourceName Resource name, used to distinguish different resources.
 * @param {number|string} id Resource ID.
 * @param {Function} repositoryExtractor Callback consuming state as a parameter and returning the
 *                                       repository object.
 * @param {Function} fetchFunction Callback implementing fetch algorithm and calling the first
 *                                 parameter passed with the result or the second with an error.
 * @param {Object} [options={}] Options.
 * @returns {Function} Function to be used as an action creator.
 */
var createFetchResource = function createFetchResource( // eslint-disable-line import/prefer-default-export
resourceName, id, repositoryExtractor, fetchFunction) {
  var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
  return function (dispatch, getState) {
    var repository = repositoryExtractor(getState());
    var resourceInRepository = (0, _repository.getResourceById)(repository, id);

    if (resourceInRepository) {
      if ((0, _resource.isReceived)(resourceInRepository) && options.ttl && !(0, _resource.isExpired)(resourceInRepository, options.ttl)) {
        dispatch({
          payload: {
            id: id,
            resourceName: resourceName
          },
          type: T.FETCH_RESOURCE_ALREADY_RECEIVED
        });
        return;
      }

      if ((0, _resource.isRequested)(resourceInRepository)) {
        dispatch({
          payload: {
            id: id,
            resourceName: resourceName
          },
          type: T.FETCH_RESOURCE_ALREADY_REQUESTED
        });
        return;
      }
    }

    dispatch({
      payload: {
        id: id,
        resourceName: resourceName
      },
      type: T.FETCH_RESOURCE_REQUESTED
    });

    var dispatchReceived = function dispatchReceived(data) {
      dispatch({
        payload: {
          data: data,
          id: id,
          resourceName: resourceName
        },
        type: T.FETCH_RESOURCE_RECEIVED
      });
    };

    var dispatchFailed = function dispatchFailed(error) {
      dispatch({
        payload: {
          error: error,
          id: id,
          resourceName: resourceName
        },
        type: T.FETCH_RESOURCE_FAILED
      });
    };

    fetchFunction(dispatchReceived, dispatchFailed);
  };
};

exports.createFetchResource = createFetchResource;